/*
 * Copyright (C) The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package aaaaawrapper;

import anywheresoftware.b4a.AbsObjectWrapper;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.IOnActivityResult;
import anywheresoftware.b4a.BA.Permissions;

import android.content.Intent;
import android.os.Bundle;
import android.app.Activity;
import android.util.Log;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.TextView;
import android.graphics.Color;

import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;

import com.google.android.gms.common.api.CommonStatusCodes;

import com.google.android.gms.samples.vision.barcodereader.R;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;
import com.google.android.gms.common.api.CommonStatusCodes;
import com.google.android.gms.samples.vision.barcodereader.ui.camera.CameraSource;
import com.google.android.gms.samples.vision.barcodereader.ui.camera.CameraSourcePreview;

import com.google.android.gms.samples.vision.barcodereader.ui.camera.GraphicOverlay;
import com.google.android.gms.vision.MultiProcessor;
import com.google.android.gms.vision.barcode.Barcode;
import com.google.android.gms.vision.barcode.BarcodeDetector;
import com.google.android.gms.samples.vision.barcodereader.BarcodeGraphicTracker;
import com.google.android.gms.samples.vision.barcodereader.BarcodeGraphic;
import com.google.android.gms.samples.vision.barcodereader.BarcodeTrackerFactory;

import com.google.android.gms.vision.Tracker;
import com.google.android.gms.vision.barcode.Barcode;

import android.view.GestureDetector;
import android.view.ScaleGestureDetector;
import android.view.MotionEvent;
import android.Manifest;
import android.content.pm.PackageManager;
import android.support.design.widget.Snackbar;
import android.content.Context;
import android.content.IntentFilter;
import android.widget.Toast;
import android.os.Build;
import android.os.Bundle;
import android.hardware.Camera;
import android.app.AlertDialog;
import android.app.Dialog;

import java.io.IOException;

import android.view.View.OnLongClickListener;

@ActivityObject
@ShortName("VisionBarcodeScanner")
@Events(values={"touch_result(data As String)", "detected_result(data As String)"})
//@Author("Github: Clayton Wilkinson, Wrapped by: Johan Schoeman")
@Version(1.00f)
@DependsOn(values={"com.android.support:support-v4", "com.android.support:appcompat-v7", "com.google.android.gms:play-services-vision", "com.android.support:design"})
@Permissions(values={"android.permission.CAMERA","android.permission.WRITE_EXTERNAL_STORAGE","android.hardware.camera","android.hardware.camera.autofocus","android.permission.VIBRATE","android.permission.FLASHLIGHT"})

public class aaaaaWrapper extends AppCompatActivity implements BarcodeGraphicTracker.BarcodeUpdateListener { //implements View.OnClickListener {
//public class androidVisionMasterWrapper implements View.OnClickListener {
	
	@Hide
	public BA ba;
	@Hide
	public String eventName;

    private static final String TAG = "Barcode-reader";

    // intent request code to handle updating play services if needed.
    private static final int RC_HANDLE_GMS = 9001;

    // permission request codes need to be < 256
    private static final int RC_HANDLE_CAMERA_PERM = 2;

    // constants used to pass extra data in the intent
    public static final String AutoFocus = "AutoFocus";
    public static final String UseFlash = "UseFlash";
    public static final String BarcodeObject = "Barcode";

    private CameraSource mCameraSource;
    private CameraSourcePreview mPreview;
    private GraphicOverlay<BarcodeGraphic> mGraphicOverlay;
	private boolean flashIsOn = false;
	
	public void Initialize(BA paramBA, String paramString) {
	    eventName = paramString.toLowerCase(BA.cul);
	    ba = paramBA;

	    String str = paramString.toLowerCase(BA.cul);
	}
	
    public void BeginScan() {
		
        ba.activity.setContentView(R.layout.barcode_capture);

        mPreview = (CameraSourcePreview) ba.activity.findViewById(R.id.preview);
        mGraphicOverlay = (GraphicOverlay<BarcodeGraphic>) ba.activity.findViewById(R.id.graphicOverlay);
		mPreview.setOnTouchListener(touchListener);
		
		mPreview.setOnLongClickListener(new OnLongClickListener() {
			public boolean onLongClick(View arg0) {
				if (flashIsOn == false) {
				  mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
				  flashIsOn = true;
				  Snackbar.make(mGraphicOverlay, "Flash/Torch turned on",
							Snackbar.LENGTH_LONG)
							.show();					  
				  
				} else {
					mCameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
					flashIsOn = false;
				    Snackbar.make(mGraphicOverlay, "Flash/Torch turned off",
							Snackbar.LENGTH_LONG)
							.show();						
				}  
				return true;    // <- set to true
			}
		});		
	
        // read parameters from the intent used to launch the activity.
        boolean autoFocus = true;
        boolean useFlash = false;

        createCameraSource(true, useFlash);

        Snackbar.make(mGraphicOverlay, "1. Touch a highlighted barcode to get its value \n2. LongClick the scanner to toggle the Flash",
                Snackbar.LENGTH_LONG)
                .show();		

        startCameraSource();		
		 
	}	
	
	public void StopScan() {
        if (mPreview != null) {
            mPreview.release();
        }		
	}
	
	private int raised = 0;	
    View.OnTouchListener touchListener = new View.OnTouchListener() {
        @Override
        public boolean onTouch(View v, MotionEvent event) {
			if (event.getActionMasked() == MotionEvent.ACTION_UP) {
				onTap(event.getRawX(), event.getRawY());	
            }
			return false;
        }
    };	
	
    /**
     * Creates and starts the camera.  Note that this uses a higher resolution in comparison
     * to other detection examples to enable the barcode detector to detect small barcodes
     * at long distances.
     *
     * Suppressing InlinedApi since there is a check that the minimum version is met before using
     * the constant.
     */
    private void createCameraSource(boolean autoFocus, boolean useFlash) {
        Context context = ba.applicationContext;

        // A barcode detector is created to track barcodes.  An associated multi-processor instance
        // is set to receive the barcode detection results, track the barcodes, and maintain
        // graphics for each barcode on screen.  The factory is used by the multi-processor to
        // create a separate tracker instance for each barcode.
        BarcodeDetector barcodeDetector = new BarcodeDetector.Builder(context).build();
        BarcodeTrackerFactory barcodeFactory = new BarcodeTrackerFactory(mGraphicOverlay, this);
        barcodeDetector.setProcessor(
                new MultiProcessor.Builder<>(barcodeFactory).build());

        if (!barcodeDetector.isOperational()) {
            // Note: The first time that an app using the barcode or face API is installed on a
            // device, GMS will download a native libraries to the device in order to do detection.
            // Usually this completes before the app is run for the first time.  But if that
            // download has not yet completed, then the above call will not detect any barcodes
            // and/or faces.
            //
            // isOperational() can be used to check if the required native libraries are currently
            // available.  The detectors will automatically become operational once the library
            // downloads complete on device.
            BA.Log("Detector dependencies are not yet available.");

            // Check for low storage.  If there is low storage, the native library will not be
            // downloaded, so detection will not become operational.
            IntentFilter lowstorageFilter = new IntentFilter(Intent.ACTION_DEVICE_STORAGE_LOW);
            boolean hasLowStorage = registerReceiver(null, lowstorageFilter) != null;

            if (hasLowStorage) {
                Toast.makeText(ba.applicationContext, R.string.low_storage_error, Toast.LENGTH_LONG).show();
                BA.Log("" + getString(R.string.low_storage_error));
            }
        }

        // Creates and starts the camera.  Note that this uses a higher resolution in comparison
        // to other detection examples to enable the barcode detector to detect small barcodes
        // at long distances.
        CameraSource.Builder builder = new CameraSource.Builder(ba.applicationContext, barcodeDetector)
                .setFacing(CameraSource.CAMERA_FACING_BACK)
                .setRequestedPreviewSize(1600, 1024)
                .setRequestedFps(15.0f);

        // make sure that auto focus is an available option
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
            builder = builder.setFocusMode(
                    autoFocus ? Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE : null);
        }

        mCameraSource = builder
                .setFlashMode(useFlash ? Camera.Parameters.FLASH_MODE_TORCH : null)
                .build();
    }	
	
    /**
     * Starts or restarts the camera source, if it exists.  If the camera source doesn't exist yet
     * (e.g., because onResume was called before the camera source was created), this will be called
     * again when the camera source is created.
     */
    private void startCameraSource() throws SecurityException {
        if (mCameraSource != null) {
            try {
                mPreview.start(mCameraSource, mGraphicOverlay);
            } catch (IOException e) {
                BA.Log("Unable to start camera source. " + e);
                mCameraSource.release();
                mCameraSource = null;
            }
        }
    }	
		
	String mybarcode = "";
    @Override
    public void onBarcodeDetected(Barcode barcode) {
		mybarcode = barcode.rawValue;
		if (ba.subExists(eventName + "_detected_result")) {
			ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_detected_result", true, new Object[] {barcode.rawValue});
		}	
    }	
	
    /**
     * onTap returns the tapped barcode result to the calling Activity.
     *
     * @param rawX - the raw position of the tap
     * @param rawY - the raw position of the tap.
     * @return true if the activity is ending.
     */
    private boolean onTap(float rawX, float rawY) {
		
        // Find tap point in preview frame coordinates.
        int[] location = new int[2];
        mGraphicOverlay.getLocationOnScreen(location);
        float x = (rawX - location[0]) / mGraphicOverlay.getWidthScaleFactor();
        float y = (rawY - location[1]) / mGraphicOverlay.getHeightScaleFactor();

        // Find the barcode whose center is closest to the tapped point.
        Barcode best = null;
        float bestDistance = Float.MAX_VALUE;
        for (BarcodeGraphic graphic : mGraphicOverlay.getGraphics()) {
            Barcode barcode = graphic.getBarcode();
            if (barcode.getBoundingBox().contains((int) x, (int) y)) {
                // Exact hit, no need to keep looking.
                best = barcode;
                break;
            }
            float dx = x - barcode.getBoundingBox().centerX();
            float dy = y - barcode.getBoundingBox().centerY();
            float distance = (dx * dx) + (dy * dy);  // actually squared distance
            if (distance < bestDistance) {
                best = barcode;
                bestDistance = distance;
            }
        }
	
        if (best != null) {
			Snackbar.make(mGraphicOverlay, best.rawValue,
					Snackbar.LENGTH_SHORT)
					.show();	
				if (ba.subExists(eventName + "_touch_result")) {
					ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_touch_result", true, new Object[] {best.rawValue});
				}
               				
			//}	
            finish();
            return true;
        }
        return false;
    }
	

}